# ShowTF


from math import sqrt
from simpleGraphicsE import *

def Nearest(a,b,x,y):
    """ Returns (dmin,kmin) where dmin is a float that is the distance
    from (a,b) to the nearest of the points (x[k].y[k]) defined
    by x and y. kmin is an int that is the index of the closest point.
    
    PreC: x and y are lists of floats that have the same length
    """
    # Start by computing the distance from (a,b) to (x[0],y[0])
    kmin = 0
    dmin = sqrt((x[0]-a)**2 + (y[0]-b)**2)
    # Now visit all the other points looking for a smaller distance
    for k in range(1,len(x)):
        dk = sqrt((x[k]-a)**2+(y[k]-b)**2)
        if dk<dmin:
            dmin = dk
            kmin = k
    return (dmin,kmin)

def Route(x,y,i):
    
    
    n = len(x)                    # Number of cities
    u = list(x)                   # A copy of x
    v = list(y)                   # A copy of y
    idx = [k for k in range(n)]   # The city index list
    uNow = u.pop(i)
    vNow = v.pop(i)
    idx.pop(i)
    itin = [i]
    odom = 0
    for k in range(n-1):
        # Get the next stop..
        (dj,j) = Nearest(uNow,vNow,u,v)
        odom = odom + dj
        uNext = u.pop(j)
        vNext = v.pop(j)
        iNext = idx.pop(j)
        itin.append(iNext)
        Connect(uNow,vNow,uNext,vNext,linecolor=YELLOW)
        uNow = uNext
        vNow = vNext
    odom = odom + sqrt((uNow-x[i])**2 + (vNow-y[i])**2)
    Connect(uNow,vNow,x[i],y[i],linecolor=YELLOW)
    return (odom,itin)

def ShowTrip(x,y,itin):
    n = len(itin)
    startCity= itin[0]
    xc = x[startCity]
    yc = y[startCity]
    M = 1.1*max(max(map(abs,x)),max(map(abs,y)))
    MakeWindow(M,label=False,bgcolor=BLACK)
    for k in range(n):
        DrawDisk(x[k],y[k],M/100.,color=CYAN)
        if k==n-1:
            nextIndex = itin[0]
        else:
            nextIndex = itin[k+1]
        xNext = x[nextIndex]
        yNext = y[nextIndex]
        Connect(xc,yc,xNext,yNext,linecolor=YELLOW)
        xc = xNext
        yc = yNext
    ShowWindow()
    
def MLB():
    """ Returns (City,x,y) where Cities is a list of strings that names
    28 MLB cities and lists of floats x and y that specify their xy location.
    """
    City = (['Anaheim','Arlington','Atlanta','Baltimore','Boston','Chicago','Cincinnati',
            'Cleveland', 'Denver','Detroit','Houston','Kansas City','Los Angeles',
            'Miami','Milwaukee','Minneapolis', 'New York', 'Oakland','Philadelphia',
            'Phoenix','Pittsburgh','San Diego','San Francisco','Seattle', 'St.Louis','Tampa',
            'Toronto','Washington'])
    x = ([-1223, -188, 451, 836, 1111, 281, 441,581,-583,516,-103, -63,-1248, 656, 271,
           1,966,-1448, 906, -938, 666, -1193, -1453,-1448, 156, 541, 696, 816])
    y = ([-289,-368,-298, 87 ,311 , 269 , 80 , 248 , 122 ,304 ,-571 , 80 ,-270 , -851 ,
         360 , 493,192 , -11 ,143 ,-312 ,171 , -368 , -11 ,675 , 45 ,-697 ,402 , 66 ])
    return (City,x,y)


if __name__ == '__main__':
    # A small test problem.
    x = [-4,-3,-2,-1,-1,-1,2,3]
    y = [3,-2,2,5,-1,-4,2,-1]
    (City,x,y)=MLB()     # When you are ready, try this.
    
    # Set up the window and display the points
    M = 1.1*max(max(map(abs,x)),max(map(abs,y)))
    MakeWindow(M,bgcolor=BLACK,labels=False)
    for k in range(len(x)):
        DrawDisk(x[k],y[k],M/50.,color=CYAN)
  
    # Generate the trip starting at city i
    i = 3
    (odom,itin) = Route(x,y,i)
    print 'i = %2d  Total Mileage = %1d'% (i,odom)
    ShowWindow()
    for k in range(len(x)):
        print City[itin[k]]
    print City[i]

    
    
 

